<?php
/**
 * Plugin Filehtml (addon for plugin Adm_redacteurs)
 * Version  : 1.6 (2021/01/03)
 * Compatibility : Guppy v5.x et v6.00.00 et Adm_redacteurs v6.14+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

require_once ADMR_HELPERS_PATH.'/manage_records_helper.php';

class Manage_filehtml_helper extends Manage_records_helper
{
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    protected $CI; // object global
    protected $_lang = array();          // Données de langue
    protected $_config = array();        // Données de configuration
    protected $_params = array();        // Données récupérés par post ou get
    protected $_sort_options = array();	 // Options de tri

    protected $_doc_infos = array('name' => 'filehtml', 'type' => 'filehtml'); // Tableau contenant les informations du document (nom, type)
    protected $_writer_rights = array(); // Droits du rédacteur
    //
    
    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */    
    public function __construct($writer_rights)
    {
        $this -> _writer_rights = $this -> _set_writer_rights($writer_rights);

        parent :: __construct();

        $this -> CI -> load_addon_helper('filehtml', 'plugin_menu_for_filehtml_helper');
        $this -> CI -> load_addon_helper('filehtml', 'filehtml_infos_helper');
                
        $this -> CI -> load_addon_model('filehtml', 'filehtml_ids_model');
        $this -> CI -> load_addon_model('filehtml', 'filehtml_menuitems_model');
        $this -> CI -> load_model('guppy_doc_model'); // Model des documents de guppy

//        $this -> CI -> load_addon_model('filehtml', 'writer_filehtml_model', array('id' => $this -> _writer_rights['id']));
        
        $this -> CI -> load_addon_lang('filehtml');
        $this -> CI -> load_addon_config('filehtml');        

        $this -> _lang = $this -> CI -> get_lang();
        $this -> _config = $this -> CI -> get_config();        

        /**
         * Controle de sécurité
         */
        if (! ADMR_IS_ADMIN && ! $this -> _is_doc_allowed()) {
            Url_helper :: redirect('display_error&type=disallowed');
        }
        //
    }

    /*
    |--------------------------------------------------------------------------
    | Methods
    |--------------------------------------------------------------------------
    */

	/**
	 * définit les options de tri des données
     * 
     * @access protected
     * @param mixed Options de tri
	 * @return void
	 */	
    protected function _set_writer_rights($writer_rights)
    {
        settype($writer_rights, 'array');
        
        return $this -> _writer_rights = $writer_rights;	// Droits du rédacteur         
    }

	// --------------------------------------------------------------------

	/**
	 * définit les options de tri des données
     * 
     * @access protected
     * @param mixed Options de tri
	 * @return void
	 */	
    protected function _set_sort_options()
    {
        return $this -> _sort_options = array('id_asc', 'id_desc', 'up_asc', 'up_desc', 'status_asc', 'status_desc');	// Options de tri         
    }

	// --------------------------------------------------------------------

	/**
	 * définit les variables utilisées dans la page 
     * 
     * @access protected
	 * @return void
	 */	
    protected function _set_params()
    {
        $this -> _params = array_merge(
            parent :: _set_params(),
            array(
                'docid'  => trim(Var_helper :: import('docid', '', NULL, array('trim'), array(array('Var_helper', 'is_valid_ids'))), '-'),
                'writerid'  => trim(Var_helper :: import('writerid', '', NULL, array('trim'), array(array('Var_helper', 'is_valid_ids'))), '-'),
            )
        );
       
        return $this -> _params;
    }

	// --------------------------------------------------------------------

	/**
	 * gére les enregistrements
     * 
     * @access public
	 * @return string code html
	 */	
    public function manage()
    {        
        parent :: manage();
    }

	// --------------------------------------------------------------------

	/**
	 * recherche des enregistrements et redirige vers la page qui va afficher les résultats de la recherche
     * 
     * @access public
	 * @return void
	 */
    public function search()
    {
        $data = array(
            'lang' => $this -> _lang,
            'content' => $this -> _build_form_data_filtering(),
        );        
        
        $this -> render_page('public_search_records_layout', $data);       
    }

	// --------------------------------------------------------------------

    /**
     * Définit et renvoie le titre de la page 
     *
     * @access public
     * @return string titre de la page
     */        
    public function set_title_page()
    {
        return ucfirst($this -> _lang['manage_filehtml']);
    }
 
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie les ajouts (js et css) dans l'entête de la page
     * 
     * @access public
	 * @return string Chaine html contenant les ajouts js et css
	 */	
    public function set_header_addons()
    {
        return parent :: set_header_addons();        
    }
   
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie la description de la page ainsi que les recommendations
     * 
     * @access public
     * @return string description de la page
	 */	
    public function set_infos_page_content()
    {		
        return $this -> _lang['infos_page_manage_filehtml'];
    }
    
	// --------------------------------------------------------------------

    /**
     * Définit le contenu du menu qui est affiché dans le bandeau en haut de la page
     * 
     * @access public
     * @return string menu de la page
	 */	
    public function set_menu_plugin_items()
    {        
        if (ADMR_IS_ADMIN) {
            return Plugin_menu_for_filehtml_helper :: set_admin_items('filehtml', $this -> _lang, $this -> _config, $this -> _writer_rights);
        } else {
            return Plugin_menu_for_filehtml_helper :: set_public_items('filehtml', $this -> _lang, $this -> _config, $this -> _writer_rights);
        }
    }

	// --------------------------------------------------------------------

    /**
     * Définit le contenu des informations du plugin qui sont affichées en bas de la page
     *
     * @return array Tableau contenant les informations du plugin sous la forme array('version' => , 'licence' => )
     */
    public function set_infos_plugin()
    {
        return Filehtml_infos_helper :: set_plugin_infos($this -> _lang);
    }

	// --------------------------------------------------------------------

	/**
	 * synchronize les données des documents enregistrées pour le rédacteur avec ceux enregistrés dans le plugin filehtml
     * 
     * @access protected
     * @param mixed Options de tri
	 * @return void
	 */	
    public function synchronize()
    {
//        $this -> CI -> writer_filehtml_model -> synchronize($this -> CI -> filehtml_ids_model); // Synchronisation des enregistrements du rédacteur avec ceux de guppy
                		
		Url_helper :: redirect('addons/filehtml/manage_filehtml');
    }

	// --------------------------------------------------------------------

	/**
	 * retourne les enregistrements
     * 
     * @access protected
	 * @return array
	 */
    protected function _get_records()
    {
        $doc_ids = empty($this -> _params['docid']) ? array() : explode('-', $this -> _params['docid']);        
        $writer_id = empty($this -> _params['writerid']) ? array() : explode('-', $this -> _params['writerid']);
        $writer_id = ADMR_IS_ADMIN ?  $writer_id : $this -> _writer_rights['id'];
        
 //       $status_options = ADMR_IS_ADMIN ?  array('a', 'i', 'd') : array('a', 'i');
        
        $records = $this -> CI -> filehtml_ids_model -> get(array('author_id' => $writer_id, 'id' => $doc_ids, 'status' => array('a', 'i')), $this -> _params['sort']);
/*
        if (ADMR_IS_ADMIN) {
            $records = $this -> CI -> filehtml_ids_model -> get(array('id' => $doc_ids, 'status' => array('a', 'i')), $this -> _params['sort']);    
        } else {
//            $records = $this -> CI -> writer_filehtml_model -> get(array('id' => $doc_ids, 'status' => array('a', 'i')), $this -> _params['sort']);
        }*/
        
        return $records;
    }

	// --------------------------------------------------------------------

	/**
	 * retourne les enregistrements
     * 
     * @access protected
	 * @return array
	 */
/*    protected function _filter_records($records)
    {
        return $records;       
    }
*/    
	// --------------------------------------------------------------------

	/**
	 * Met à jour des données des enregistrements postés
     * 
     * @access public
	 * @return boolean
	 */	
/*    protected function _update_records()
    {
        return parent :: _update_records();
    }
*/
	// --------------------------------------------------------------------

	/**
	 * définit les options du menu de sélection des actions pour la mise à jours des enregistrements sous la forme 
     * array(
     *    array('label' => '', 'value' => '', 'attrs' => array()),
     * )
     * 
     * @access protected
	 * @return array
	 */	
    protected function _set_select_actions_for_update_records()
    {
        $actions = array();
      
        // (Dés)Activation des enregistrements
        if (!empty($this -> _writer_rights['filehtml_act'])) {
            $actions[] = array('label' => ucfirst($this -> _lang['activate_docs_selected']),   'value' => 'a');
            $actions[] = array('label' => ucfirst($this -> _lang['inactivate_docs_selected']), 'value' => 'i');            
        }

        // Suppression des enregistrements
        if (!empty($this -> _writer_rights['filehtml_del'])) {
            $actions[] = array('label' => ucfirst($this -> _lang['delete_docs_selected']), 'value' => 'd', 'attrs' => array());
        }

        // Remise à zéro des compteurs
        if (!empty($this -> _writer_rights['filehtml_reset_counter'])) {
            $actions[] = array('label' => ucfirst($this -> _lang['reset_counter_of_docs_selected']), 'value' => 'reset_counter', 'attrs' => array());
        }

        return $actions;
    }

	// --------------------------------------------------------------------

	/**
	 * Met à jour les données des enregistrements selon l'action sélectionnée
     * 
     * @access protected
     * @param array Identifiants des enregistrements à mettre à jour
     * @param string Identifiant de l'action à appliquer pour la mise à jour
	 * @return boolean
	 */
    protected function _update_records_by_action($records_ids, $action)
    {
        // (Dés)Activation des enregistrements
        if (in_array($action, array('a', 'i'))) {
            return $this -> _activate_records_or_not($records_ids, $action);
        }
        // Suppression des enregistrements
        elseif ($action == 'd') { 
            return $this -> _delete_records($records_ids);
        }        
        // Suppression des enregistrements
        elseif ($action == 'reset_counter') { 
            return $this -> _reset_counter_of_records($records_ids);
        }        
        //           
        
        return FALSE;
    }
    
	// --------------------------------------------------------------------

	/**
	 * remet à zéro les compteurs des enregistrements sélectionnés
     * 
     * @access public
     * @param array Tableau contenant les identifiants des enregistrements
	 * @return boolean
	 */
    protected function _reset_counter_of_records($records_ids)
    {      
        if (!empty($this -> _writer_rights['filehtml_reset_counter'])) {
            $this -> CI -> load_addon_model('filehtml', 'filehtml_counters_model');
            
            foreach($records_ids as $record_id) {
                $this -> CI -> filehtml_counters_model -> load_counter($record_id);
                $this -> CI -> filehtml_counters_model -> reset_counter();    
            }
            
            return TRUE;
        }
        
        return FALSE;
    }

	// --------------------------------------------------------------------

	/**
	 * Active ou désactive les enregistrements sélectionnés
     * 
     * @access public
     * @param array Tableau contenant les identifiants des enregistrements
     * @param string action à appliquer (a = activation et i = désactivation)
	 * @return boolean
	 */
    protected function _activate_records_or_not($records_ids, $action)
    {
        if (!empty($this -> _writer_rights['filehtml_act'])) {
            $this -> CI -> filehtml_ids_model -> update_status_by_id($records_ids, $action);
            $this -> _update_status_for_menuitems_linked_to_records($records_ids, $action);

         //   $this -> CI -> writer_filehtml_model -> update_status_by_id($records_ids, $action);           
            return TRUE;
        }
        
        return FALSE;
    }
    
	// --------------------------------------------------------------------

	/**
	 * supprime les enregistrements sélectionnés
     * 
     * @access public
     * @param array Tableau contenant les identifiants des enregistrements
	 * @return boolean
	 */
    protected function _delete_records($records_ids)
    {      
        if (!empty($this -> _writer_rights['filehtml_del'])) {
            $this -> CI -> filehtml_ids_model -> update_status_by_id($records_ids, 'd');
            $this -> _update_status_for_menuitems_linked_to_records($records_ids, 'd');

//            $this -> CI -> filehtml_menuitems_model -> delete_by_filehtml_id($records_ids);

       //     $this -> CI -> writer_filehtml_model -> delete_by_id($records_ids);
            return TRUE;
        }
        
        return FALSE;
    }
    
	// --------------------------------------------------------------------

	/**
	 * Met à jours le status des items de menu associés aux enregistrements sélectionnés
     * 
     * @access public
     * @param array Tableau contenant les identifiants des enregistrements
	 * @return boolean
	 */
    protected function _update_status_for_menuitems_linked_to_records($records_ids, $status)
    {      
        $menuitems = $this -> CI -> filehtml_menuitems_model -> get(array('id_filehtml' => $records_ids));

        return $this -> CI -> guppy_doc_model -> update_status_by_id(array_keys($menuitems), $status); // Documents de guppy
    }
    
	// --------------------------------------------------------------------

    /*
     * Affiche ou renvoie le contenu de la vue
     *
	 * @access protected
     * @param array Données des enregistrements
     * @param boolean Renvoie le contenu (TRUE) ou affiche (FALSE) de la vue
     * @return string|void
     */
    protected function _build_view($records)
    {
        return parent :: _build_view($records);
    }    

	// --------------------------------------------------------------------

    /**
	 * Définit et retourne le layout de la vue
     * 
	 * @access protected
     * @return string nom du fichier qui définit le layout
	 */	
    protected function _get_layout_for_manage_records_view()
    {
        return parent :: _get_layout_for_manage_records_view();   
    }

	// --------------------------------------------------------------------

	/**
	 * construit le formulaire de sélection des données
     * 
     * @access protected
	 * @return string Formulaire html
	 */	
    protected function _build_form_data_filtering()
    {
        return '';
    }

	// --------------------------------------------------------------------

	/**
	 * construit le tableau de données
     * 
	 * @return string Tableau HTML contenant les enregistrements
	 * @access public
	 */	
    protected function _build_data_table($records)
    {		
        return parent :: _build_data_table($records);        
    }

	// --------------------------------------------------------------------
  
	/**
	 * ajoute des données supplémentaires à l'enregistrement
     * 
	 * @access protected
     * @param array Tableau de données initiales de l'enregistrement
	 * @return array Tableau contenant les nouvelles données de l'enregistrement
	 */    
    protected function _add_more_data_for_record($record)
    {
        return $this -> CI -> filehtml_ids_model -> get_by_id($record['id']);        
    }
    
	// --------------------------------------------------------------------
  
	/**
	 * Définit et renvoie les entêtes de la table de données
     * 
	 * @access protected
	 * @return array Tableau contenant les entêtes du tableau de données
	 */    
    protected function _set_data_table_headers()
    {
		$headers = array(
            parent :: _set_data_table_header_for_id(),
			parent :: _set_data_table_header_for_description($this -> _lang['title']),
            parent :: _set_data_table_header_for_status(),
            ADMR_IS_ADMIN ? parent :: _set_data_table_header_for_state() : array(),
            parent :: _set_data_table_header_for_commands(),
		);

        return $headers;
    }

	// --------------------------------------------------------------------
  
	/**
	 * Définit et renvoie le contenu des cellules de la table de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return array Tableau contenant le contenu des cellules de la table de données
	 */    
    protected function _set_data_table_cells($record)
    {
    	$cells = array(
            parent :: _set_data_table_cell_for_id($record),
            parent :: _set_data_table_cell_for_description($record),
            parent :: _set_data_table_cell_for_status($record),
            ADMR_IS_ADMIN ? parent :: _set_data_table_cell_for_state($record) : array(),
            parent :: _set_data_table_cell_for_commands($record),
    	);
        
        return $cells;
    }

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Id" du tableau de données
     * 
	 * @access private
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_id($record)
	{        
		return parent :: _add_contents_for_td_id($record);
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu à la cellule "Description" du tableau de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_description($record)
	{
        if (!empty($this -> _writer_rights['filehtml_edit'])) {
            return parent :: _add_contents_for_td_description($record);
        } else {
            return $this -> _get_text_for_td_description($record);
        }
	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie le texte de la cellule intitulée "Description"
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
    protected function _get_text_for_td_description($record)
    {
        return $this -> CI -> filehtml_ids_model -> set_doc_heading($record);      
    }

	// --------------------------------------------------------------------

	/**
	 * Renvoie l'url du lien du document dans la cellule intitulée "Description"
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Url du lien
	 */
    protected function _build_href_for_td_description($record)
    {
		$base_href = (ADMR_IS_ADMIN) ? CHEMIN.'admin/'.ADMR_BASE_HREF_PLUGIN : ADMR_BASE_HREF_PLUGIN;

        return $base_href.'&ctrl=addons/filehtml/add_filehtml&id='.$record['id'];       
    }

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Etat" du tableau de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_state($record)
	{
		return parent :: _add_contents_for_td_state($record);
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Status" du tableau de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_status($record)
	{
		return parent :: _add_contents_for_td_status($record);
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Commandes" du tableau de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_commands($record)
	{
        $id = $record['id'];
       
        $view = '';
        
        // Prévisualisation
        $view .= Html_helper :: addA(
			$href = ADMR_SITE_URL.'plugins/filehtml/filehtml.php?lng='.ADMR_LNG.'&amp;pg='.$id,
			$contents = Html_helper :: addImg(
				$path = ADMR_IMG_LOOK,
				$alt = ucfirst($this -> _lang['view']),
				$attr = array(
					'title' => $alt,
					'border' => '0'
				)
			)
            .'&nbsp;&nbsp;',
			$attr = array('target' => '_blank')
		);

        if (!empty($this -> _writer_rights['filehtml_menuitems_manage'])) {

            // Ajout d'un item de menu associé au document
            if (!empty($this -> _writer_rights['filehtml_menuitems_add'])) {
                $view .= Html_helper :: addA(
        			$href = ADMR_BASE_HREF_PLUGIN.'&amp;ctrl=addons/filehtml/add_filehtml_menuitems&amp;itemdocid='.$id,
        			$contents = Html_helper :: addImg(
        				$path = ADMR_IMG_ADD_FILE,
        				$alt = ucfirst($this -> _lang['add_menu_item_for_document']),
        				$attr = array(
        					'title' => $alt,
        					'border' => '0'
        				)
        			)
                    .'&nbsp;&nbsp;',
        			$attr = array()
        		);
            }
        	
            // Gestion des items de menu associés au document
            $nb_menuitems = $this -> CI -> filehtml_menuitems_model -> count(array('id_filehtml' => $id));

    //        if (!empty($nb_menuitems)) {           
                $view .= Html_helper :: addA(
        			$href = ADMR_BASE_HREF_PLUGIN.'&amp;ctrl=addons/filehtml/manage_filehtml_menuitems&amp;sort='.$this -> _config['sort_option_by_default'].'&amp;itemdocid='.$id,
        			$contents = Html_helper :: addImg(
        				$path = ADMR_IMG_EXPLORER,
        				$alt = ucfirst($this -> _lang['manage_all_menu_items_for_document']),
        				$attr = array(
        					'title' => $alt,
        					'border' => '0'
        				)
        			).'&nbsp;('.$nb_menuitems.')',
        			$attr = array()
        		);
    //        }
        }		  
        
        return $view;
	}

	// --------------------------------------------------------------------

    /**
	 * Vérifie la validité d'accès au document par le rédacteur
     * 
	 * @access protected
     * @return boolean
	 */
    protected function _is_doc_allowed()
    {   
        if (!empty($this -> _writer_rights[$this -> _doc_infos['name'].'_manage'])) // Autorisation d'accès à la section du document
        { 
            return TRUE;
        }
        
        return FALSE;    
    }

	// --------------------------------------------------------------------

} //End of Class