<?php
/**
 * Plugin Filehtml (addon for plugin Adm_redacteurs)
 * Version  : 1.6 (2021/01/03)
 * Compatibility : Guppy v5.x et v6.00.00 et Adm_redacteurs v6.14+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

require_once ADMR_HELPERS_PATH.'/plugin_menu_helper.php';

class Plugin_menu_for_filehtml_helper extends Plugin_menu_helper
{
    /**
	 * Définit et retourne les items du menu qui est affiché dans le bandeau en haut de la page coté public
     * 
	 * @access public
	 * @param string Nom du document
	 * @param array Tableau contenant les traduction de langue
	 * @param array Tableau contenant les options de configuration
     * @param array Tableau contenant les paramètres des droits du rédacteur
     * @param array Paramètres optionnels
	 * @return array
	 */
	public static function set_public_items($doc_name, $lang, $config, $writer_rights = array(), $params = array())
	{
        return array(
            parent :: add_item_for_go_dashboard($doc_name, $lang),
            parent :: add_item_for_addon_manage_docs($doc_name, $lang, $writer_rights),
            parent :: add_item_for_addon_add_doc($doc_name, $lang, $writer_rights),
            parent :: add_item_for_addon_search_docs($doc_name, $lang),
            parent :: add_item_for_disconnect($doc_name, $lang)
        );
    }

	// --------------------------------------------------------------------

    /**
	 * Définit et retourne les items du menu qui est affiché dans le bandeau en haut de la page coté admin
     * 
	 * @access public
	 * @param string Nom du document
	 * @param array Tableau contenant les traduction de langue
	 * @param array Tableau contenant les options de configuration
     * @param array Tableau contenant les paramètres des droits du rédacteur
     * @param array Paramètres optionnels
	 * @return array
	 */
	public static function set_admin_items($doc_name, $lang, $config, $writer_rights = array(), $params = array())
    {
        return array(
            parent :: add_item_for_go_dashboard($doc_name, $lang),
            parent :: add_item_for_manage_docs($doc_name, $lang, $writer_rights, $params = array(
                'pattern_to_search_in_uri' => array('method' => 'strict', 'string' => 'addons/'.$doc_name.'/manage_'.$doc_name),
                'addon_dir_path' => 'addons/'.$doc_name.'/', // Chemin du répertoire de l'addon
            )),
            parent :: add_item_for_add_doc($doc_name, $lang, $writer_rights, $params = array(
                'pattern_to_search_in_uri' => array('method' => 'strict', 'string' => 'addons/'.$doc_name.'/add_'.$doc_name),
                'addon_dir_path' => 'addons/'.$doc_name.'/', // Chemin du répertoire de l'addon
            )),
            parent :: add_item_for_manage_docs($doc_name.'_menuitems', $lang, $writer_rights, $params = array(
                'pattern_to_search_in_uri' => array('method' => 'strict', 'string' => 'addons/'.$doc_name.'/manage_'.$doc_name.'_menuitems'),
                'addon_dir_path' => 'addons/'.$doc_name.'/', // Chemin du répertoire de l'addon
            )),
            parent :: add_item_for_add_doc($doc_name.'_menuitems', $lang, $writer_rights, $params = array(
                'pattern_to_search_in_uri' => array('method' => 'strict', 'string' => 'addons/'.$doc_name.'/add_'.$doc_name.'_menuitems'),
                'addon_dir_path' => 'addons/'.$doc_name.'/', // Chemin du répertoire de l'addon
            )),
            parent :: add_item_for_addon_manage_docs_deleted($doc_name, $lang),  // Gestion des documents supprimés          
            parent :: add_item_for_addon_check_docs_integrity($doc_name, $lang), // Intégrité des données des documents
            parent :: add_item_for_addon_search_docs($doc_name, $lang),          // Recherche de documents
        );
    }

	// --------------------------------------------------------------------

} // End of class