<?php
/**
 * Plugin Filehtml (addon for plugin Adm_redacteurs)
 * Version  : 1.6 (2021/01/03)
 * Compatibility : Guppy v5.x et v6.00.00 et Adm_redacteurs v6.14+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

class Filehtml_ids_model
{       
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    protected $_DB_file_path = FILEHTML_IDS_DB;    // Chemin du fichier contenant la base de données
    protected $_DB_counter_path = FILEHTML_NEXTID_DB; // Chemin du fichier contenant l'id du prochain document
    protected $_data_DB = array();    //Données de la base de données

    protected $_params = array(
        'id'     => 0, // Identifiant du document        
        'status' => '', // Status du document
        'type'   => 'filehtml', // Type du document
        'title1' => '', // Titre du document (langue 1)
        'title2' => '', // Titre du document (langue 2)
        'author_id' => 0, // Id de l'auteur du document (0 = admin)
        'mode'   => 0   // 1 : new, 2 : updated, 0 : approved 
    );

    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */
    public function __construct()
	{
        if (is_file($this -> _DB_file_path)) {
            $this -> _data_DB = $this -> read();    
        }       
	}

	// --------------------------------------------------------------------
 
	/**
	 * récupére les données des rédacteurs enregistrés (tous par défaut)
     * 
	 * @access public
     * @param array Tableau de filtres (optionnel) sous la forme array($key1 => mixed $values1, $key2 => mixed $values2, ...)
     * @param string Option de tri (optionnel)
	 * @return array tableau contenant les données des rédacteurs enregistrés
	 */
	function get($filters = array(), $sort_by = '')
	{
        settype($filters, 'array');
        $data = array();
      
        if (!empty($filters) ) {
            $data = $this -> get_many_by_keys($filters);
        }
        else {
            $data = $this -> get_all();
        }
        
        if (!empty($sort_by)) {
            $data = $this -> sort_by_key($data, $sort_by);
        }
//print_r($data);        
        return $data;
	}

	// --------------------------------------------------------------------

    /**
     * récupére tous les documents enregistrés
     * 
     * @access public
     * @return array
     */
    function get_all()
    {  
        return $this -> _data_DB;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre de documents enregistrés
     * 
     * @access public
     * @param  string Type du document (optionnel)
     * @return integer
     */
	function count($filters = array('type' => NULL, 'author' => NULL))
	{
        if (isset($filters['type'])) {
            return $this -> count_by_type($type);
        }
        else if (isset($filters['author'])) {
            return $this -> count_by_author($author);
        }
        else {
            return $this -> count_all();
        }
 	}

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre total de documents enregistrés
     * 
     * @access public
     * @return integer
     */
    function count_all()
    {
        return count($this -> _data_DB);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre total de documents enregistrés dans un type donné
     * 
     * @access public
     * @param  string Type du document
     * @return integer
     */
	function count_by_type($type)
	{
        return count($this -> get_many_by_type($type));
 	}

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre total de documents enregistrés par un auteur
     * 
     * @access public
     * @param  string Nom de l'auteur des documents
     * @return integer
     */
	function count_by_author($author)
	{
        return count($this -> get_many_by_author($author));
 	}

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa valeur
     * 
     * @param array Données de la base
     * @param string Clé
     * @param string Valeur de la clé
     * @return array
     */
    function select_by_key($data_DB = array(), $key = '', $value = '')
    {
        settype($data_DB, 'array');
        settype($value, 'string');
        
        foreach($data_DB as $row) {
            if (isset($row[$key]) && $row[$key] == $value) {
                return $row;
            }
        }    
        
        return array();
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa ou ses valeurs
     * 
     * @param array Données de la base
     * @param string Clé
     * @param string|array Valeur(s) de la clé
     * @return array
     */
    function select_many_by_key($data_DB = array(), $key = '', $value = array())
    {
        settype($data_DB, 'array');
        settype($value, 'array');
        $data = array();
        
        foreach($data_DB as $row) {
            if (isset($row[$key]) && in_array($row[$key], $value)) {
                $data[$row['id']] = $row;
            }
        }    
        
        return $data;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa ou ses valeurs
     * 
     * @param array Données de la base
     * @param array Tableau contenant les critères de sélection sous la forme array($key1 => mixed $values1, $key2 => mixed $values2,...)
     * @return array
     */
    function select_many_by_many_keys($data_DB = array(), $keys = array())
    {
        settype($data_DB, 'array');
        settype($keys, 'array');

        
        $data = $data_DB;

        foreach($keys as $key => $values) {
            
            if (empty($values)) {
                continue;    
            }
            
            settype($values, 'array');
            
            foreach($data as $id => $row) {
                if (!isset($row[$key]) || !in_array($row[$key], $values)) {
                    unset($data[$id]);
                }
            }           
        }        
        
        return $data;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données du document identifié par son identifiant
     * 
     * @param integer|array Id du ou des rédacteur(s))
     * @return array Données du rédacteur retourné sous la forme array('name' => , 'id' => , 'status' => , 'groupid' => )
     */
    function get_by_id($id)
    {
        return $this -> select_by_key($this -> _data_DB, 'id', $id);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des document(s) identifié(s) par leurs identifiants
     * 
     * @param integer|array Id du ou des rédacteur(s))
     * @return array Données des documents retourné sous la forme
     *                     array( 
     *                          $id => array('name' => , 'id' => , 'status' => , 'groupid' => ),
     *                          $id => array('name' => , 'id' => , 'status' => , 'groupid' => ), 
     *                      )
     */
    function get_many_by_id($ids)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'id', $ids);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des document(s) identifié(s) par leur type
     * 
     * @param string|array type du ou des documents(s))
     * @return array Données des documents retourné sous la forme
     *                     array( 
     *                          $id => array('id' => , 'status' => , 'type' =>  , 'title' => '', 'comment' => '', 'author' => ''),
     *                          $id => array('id' => , 'status' => , 'type' =>  , 'title' => '', 'comment' => '', 'author' => ''),
     *                      )
     */
    function get_many_by_type($types)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'type', $types);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des documents identifiés par le status
     * 
     * @param string Status du rédacteur
     * @return array
     */
    function get_many_by_status($status)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'status', $status);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des documents identifiés par le status
     * 
     * @param string Nom de l'auteur
     * @return array
     */
    function get_many_by_author($author)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'author', $author);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des documents identifiés par les critères de sélection
     * 
     * @param array Tableau contenant les critères de sélection sous la forme array( key => value)
     * @return array
     */
    function get_many_by_keys($keys)
    {
        settype($keys, 'array');

        return $this -> select_many_by_many_keys($this -> _data_DB, $keys);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données du document identifié par le status différent du sien
     * 
     * @param string Status différent de celui du document
     * @return array
     */
    function get_many_by_not_status($status)
    {
        return array_diff_key($this -> _data_DB, $this -> select_many_by_key($this -> _data_DB, 'status', $status));
    }

	// --------------------------------------------------------------------

    /**
     * récupére les données brutes de la table indexée des documents (id||status||type||title||||author||mode)
     *
     * @return array
     */
    protected function read()
    {
        $data_DB = array();
        $records = array();
        
        if (is_file($this -> _DB_file_path)) {
            $data_DB = file($this -> _DB_file_path);
    		
            foreach($data_DB as $data) {
                list($id, $status, $type, $title1, $title2, $author_id, $mode) = explode(CONNECTOR, trim($data));
    			
                $records[$id] = array(
                    'id'     => $id,      // Identifiant du document
                    'status' => $status,  // Status du document
                    'type'   => $type,    // Type du document
                    'title1' => $title1,   // Titre du document (lang 1)
                    'title2' => $title2,   // Titre du document (lang 2)
                    'author_id' => $author_id,   // Identifiant de l'auteur du document
                    'mode'   => $mode,   // Titre du document
                );
            }
        }
        
        return $records;
    }

	// --------------------------------------------------------------------

    /**
     * Efface le contenu de la table et le remplace avec les données passées en paramètre
     * 
     * @param array Données à écrire
     * @return array
     */
    protected function write()
    {   
        $data = $this -> _data_DB;

        $file_path = $this -> _DB_file_path;
        $data_out = '';
        
        foreach($data as $row) {     
            $data_out .= implode(CONNECTOR, array_values($row))."\n";
        }

        $fhandle = fopen($file_path, 'w');
        fputs($fhandle, $data_out);
        fclose($fhandle);
        
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * Ajoute un enregistrement dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param array Données de l'enregistrement du rédacteur dans la table
     * @return integer Id du compteur
     */
    function insert($data)
    {
        if (empty($data)) {
            return FALSE;
        }
        
//        $data = $this -> _clean($data);
        $data['id'] = $this -> update_counter_id();  // Mise à jour du compteur id
        $data['mode'] = ADMR_IS_ADMIN ? 0 : 1;

        $this -> _data_DB[$data['id']] = array_merge($this -> _params, $this -> _clean($data), array('id' => $data['id']));
        
        $this -> write();

        return $data['id'];
    }

	// --------------------------------------------------------------------

    /**
     * Ajoute plusieurs enregistrements dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param array Tableau de données des enregistrements des documents dans la table 
     *              sous la forme array(
     *                                array('id0','status0','type0', 'title0', 'comment0', 'author0'),
     *                                array('id1','status1','type1', 'title1', 'comment1', 'author1'),
     *                            )
     * @return integer Id du compteur
     */
    function insert_many($docs)
    {
        if (empty($docs)) {
            return FALSE;
        }

        $counter = $this -> get_counter_id(); // id du compteur

        foreach($docs as $doc) {
            $counter++;           
            $this -> _data_DB[$counter] = array_merge($this -> _params, $this -> _clean($doc), array('id' => $counter));
        }
       
        $this -> write();
        
        $this -> write_counter_id($counter);  // compteur

        return $counter;
    }
    
	// --------------------------------------------------------------------

    /**
     * Met à jour l'enregistrement identifié par le paramètre id 
     * dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param array Données de l'enregistrement du document à mettre à jours dans la table
     * @return array
     */
    function update($data)
    {
        if (empty($data) || empty($data['id']) || !isset($this -> _data_DB[$data['id']])) {
            return FALSE;
        }
        
        $data['mode'] = ADMR_IS_ADMIN ? 0 : 2;
        
        $this -> _data_DB[$data['id']] = array_merge($this -> _params, $this -> _clean($data));
        
        $this -> write();

        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour plusieurs enregistrements dans la table indexée des documents (id||status||type||title1||title1||author_id||mode)
     * 
     * @param array Tableau de données des enregistrements des documents dans la table 
     *              sous la forme array(
     *                                array('id' => , 'status' => , 'type' => , 'title1' => , 'title2' => , 'author_id' => , 'mode' => ),
     *                                array('id' => , 'status' => , 'type' => , 'title1' => , 'title2' => , 'author_id' => , 'mode' => ),
     *                            )
     * @return
     */
    function update_many($docs)
    {
        if (empty($docs)) {
            return FALSE;
        }

        foreach($docs as $doc) {          
            $this -> _data_DB[$doc['id']] = array_merge($this -> _params, $this -> _clean($doc));
        }
       
        ksort($this -> _data_DB);
        
        $this -> write();

        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * supprime les documents identifiés par les identifiants dans la table
     * 
     * @param integer|array Identifiants des documents dans la table
     * @return array
     */
    function delete_by_id($ids)
    {
        settype($ids, 'array');
        
        if (empty($ids)) {
            return FALSE;
        }
        
        $db =& $this -> _data_DB;
        
        foreach($db as $key => $row){
            if (in_array($row['id'], $ids)) {
                unset($db[$key]);
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données du rédacteur
     * 
     * @access private 
	 * @param array Données du rédacteur à nettoyer
	 * @return array
	 */        
    private function _clean($data)
    {
        settype($data, 'array');
        
        return array_map('RemoveConnector', array_intersect_key($data, $this -> _params));
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour l'enregistrement identifié par le paramètre id 
     * dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param integer Id du document dans la table
     * @param array Données de l'enregistrement du document dans la table
     * @return array
     */
/*    function update_by_id($id, $data)
    {
        if (isset($this -> _data_DB[$id]) && !empty($data)) {  
            $this -> _data_DB[$id] = array(
                'id'      => $id,      // Identifiant du document
                'status'  => $data['status'],  // Status du document
                'type'    => $data['type'],    // Type du document
                'title'   => $data['title'],   // Titre du document
                'comment' => $data['comment'], // commentaire
                'author'  => $data['author'],  // Auteur 
            );            
            
            $this -> write();
            
            return TRUE;
        }
        
        return FALSE;
    }
*/
	// --------------------------------------------------------------------

    /**
     * Renvoie le compteur de l'id des enregistrements
     *
     * @return integer Identifiant du compteur
     */
    function get_counter_id()
    {
        $counter = $this -> _DB_counter_path;
        
        $id = ReadCounter($counter);
        
        return $id;
    }

	// --------------------------------------------------------------------

    /**
     * Renvoie le compteur de l'id des enregistrements
     *
     * @return integer Identifiant du compteur
     */
    function write_counter_id($id)
    {
        $counter = $this -> _DB_counter_path;
        
        WriteCounter($counter, $id);
        
        return $id;

    }

	// --------------------------------------------------------------------

    /**
     * Met à jour le compteur de l'id des enregistrements et renvoie le nouvel identifiant
     *
     * @return integer Identifiant du compteur
     */
    function update_counter_id()
    {
        $counter = $this -> _DB_counter_path;
        
        $id = ReadCounter($counter);
        $id++;
        WriteCounter($counter, $id);
        
        return $id;
    }

	// --------------------------------------------------------------------

    /**
     * Renvoie l'id enregistré dans le compteur
     *
     * @return integer id
     */
    function get_last_insert_id()
    {
        $counter = $this -> _DB_counter_path;
        
        return ReadCounter($counter);
    }

	// --------------------------------------------------------------------

    /**
     * Tri les enregistrements
     * 
     * @param string Option de tri
     * @return array Données triées
     */
    function sort_by_key($data, $sort_option)
    {       
        if(empty($data)) {
            return $data;
        }

        // Tableau en colonne
        foreach ($data as $key => $row) {
            $id[$key]     = $row['id'];
            $title[$key]   = $row['title1'];
            $status[$key] = $row['status'];
//            $type[$key]   = $row['type'];
            $author[$key] = $row['author_id'];
            $mode[$key]   = $row['mode'];
        }
        //
        
        switch ($sort_option) {
			case 'id_desc' :
				array_multisort($id, SORT_NUMERIC, SORT_DESC, $data);
				break;
			case 'id_asc' :
				array_multisort($id, SORT_NUMERIC, SORT_ASC, $data);
				break;
/*			case 'type_asc' :
				array_multisort($type, SORT_STRING, SORT_ASC, $data);
				break;
			case 'type_desc' :
				array_multisort($type, SORT_STRING, SORT_DESC, $data);
				break;*/
			case 'author_asc' :
				array_multisort($author, SORT_NUMERIC, SORT_ASC, $data);
				break;
			case 'author_desc' :
				array_multisort($author, SORT_NUMERIC, SORT_DESC, $data);
				break;
			case 'ch_asc' :
				array_multisort(array_map('strtolower', $title), SORT_STRING, SORT_ASC, $data);
				break;
			case 'ch_desc' :
				array_multisort(array_map('strtolower', $title), SORT_STRING, SORT_DESC, $data);
				break;
			case 'status_asc' :
				array_multisort($status, SORT_ASC, $data);
				break;
			case 'status_desc' :
				array_multisort($status, SORT_DESC, $data);
				break;
			case 'up_asc' :
				array_multisort($mode, SORT_NUMERIC, SORT_ASC, $data);
				break;
			case 'up_desc' :
				array_multisort($mode, SORT_NUMERIC, SORT_DESC, $data);
				break;
			default :
				array_multisort($id, SORT_NUMERIC, SORT_DESC, $data);
		}	
	
		return $data;
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour les status des enregistrement dans la table 
     * 
     * @param array Id des enregistrements dans la table
     * @param string Status à affecter
     * @return array
     */
    function update_status_by_id($ids, $status)
    {
        settype($ids, 'array');
        $db =& $this -> _data_DB;
                
        foreach($db as $key => $row){
            if (in_array($row['id'], $ids)) {
                $db[$key]['status'] = $status;   
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour les status des enregistrement dans la table 
     * 
     * @param array type des documents
     * @param string Status à affecter
     * @return array
     */
    function update_status_by_type($types, $status)
    {
        settype($types, 'array');
        $db =& $this -> _data_DB;
                
        foreach($db as $key => $row){
            if (in_array($row['type'], $types)) {
                $db[$key]['status'] = $status;   
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour le mode des documents dans la table 
     * 
     * @param array Id des enregistrements dans la table
     * @param string Status à affecter
     * @return array
     */
    function update_mode_by_id($ids, $mode)
    {
        settype($ids, 'array');
        $db =& $this -> _data_DB;
                
        foreach($db as $key => $row){
            if (in_array($row['id'], $ids)) {
                $db[$key]['mode'] = $mode;   
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour le mode des documents dans la table 
     * 
     * @param array Id des enregistrements dans la table
     * @param string Id de l'auteur à affecter
     * @return array
     */
    function update_author_by_id($ids, $author_id)
    {
        settype($ids, 'array');
        $db =& $this -> _data_DB;
                
        foreach($db as $key => $row){
            if (in_array($row['id'], $ids)) {
                $db[$key]['author_id'] = $author_id;   
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------
 
	/**
	 * Définit et renvoie l'intitulé du document
     * 
     * @access public 
	 * @param array Tableau contenant les paramètres du document
	 * @return string Intitulé descriptif du document
	 */
    public function set_doc_heading($params)
    {        
        global $lng, $lang;
        
        $title = ($lng == $lang[1]) ? $params['title2'] : $params['title1'];
        
        $title = strip_tags($title);
        
        if (strlen($title) > 142) {
            $title = CutLongWord($title, 142).' .../...';
        }
        
        return $title; 
    }

	// --------------------------------------------------------------------

} // End of Class